import ij.*;
import ij.plugin.filter.*;
import ij.process.*;
import ij.gui.*;
import ij.measure.*;
import java.awt.*;

/** This plugin generates and displays the histogram of a stack. */
public class Stack_Histogram implements PlugInFilter, Measurements {
	ImagePlus imp;

	public int setup(String arg, ImagePlus imp) {
		if (IJ.versionLessThan("1.21a"))
			return DONE;
		this.imp = imp;
		return DOES_ALL+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		new StackHistogramWindow(imp);
	}

}


class StackHistogramWindow extends HistogramWindow {

	StackHistogramWindow(ImagePlus imp) {
		super(imp);
	}
	
	public void showHistogram(ImagePlus imp, int bins) {
		setup();
		int type = imp.getType();
		boolean fixedRange = type==ImagePlus.GRAY8 || type==ImagePlus.COLOR_256 || type==ImagePlus.COLOR_RGB;
		calculateStackHistogram(imp, fixedRange, bins);
		lut = imp.createLut();
		img = this.imp.getImage();
		cTable = imp.getCalibration().getCTable();
		Graphics g = img.getGraphics();
		if (g!=null)
			{drawHistogram(g, fixedRange); g.dispose();}
		this.imp.setImage(img); // needed to get text to show on WinNT??
		this.imp.draw();
	}

	public void calculateStackHistogram(ImagePlus imp, boolean fixedRange, int bins) {
		ImageStack stack = imp.getStack();
		int size = stack.getSize();
		Calibration cal = imp.getCalibration();
		stats = imp.getStatistics(AREA+MEAN+MODE+MIN_MAX,bins);
		histogram = new int[stats.nBins];
		double v;
		double sum = 0;
		double sum2 = 0;
		ImageProcessor ip = imp.getProcessor();
		int width, height;
		int rx, ry, rw, rh;
		double pw, ph;
		
		width = ip.getWidth();
		height = ip.getHeight();
		Rectangle roi = ip.getRoi();
		if (roi != null) {
			rx = roi.x;
			ry = roi.y;
			rw = roi.width;
			rh = roi.height;
		} else {
			rx = 0;
			ry = 0;
			rw = width;
			rh = height;
		}
		
		pw = 1.0;
		ph = 1.0;
		stats.roiX = rx*pw;
		stats.roiY = ry*ph;
		stats.roiWidth = rw*pw;
		stats.roiHeight = rh*ph;
		
		// Find min and max
		int[] mask = imp.getMask();
		double roiMin = Double.MAX_VALUE;
		double roiMax = -Double.MAX_VALUE;
		for (int slice=1; slice<=size; slice++) {
			IJ.showStatus("MinMax: "+slice+"/"+size);
			IJ.showProgress((double)slice/size);
			ip = stack.getProcessor(slice);
			for (int y=ry, my=0; y<(ry+rh); y++, my++) {
				int i = y * width + rx;
				int mi = my * rw;
				for (int x=rx; x<(rx+rw); x++) {
					if (mask==null || mask[mi++]==ip.BLACK) {
						v = ip.getPixelValue(x,y);
						if (v<roiMin) roiMin = v;
						if (v>roiMax) roiMax = v;
					}
					i++;
				}
			}
			stats.min = roiMin; stats.max = roiMax;
		}
		if (fixedRange) {
			stats.histMin = 0;
			stats.histMax = 255;
		} else {
			stats.histMin = stats.min; 
			stats.histMax =  stats.max;
		}
		
		// Generate histogram
		double scale = ( stats.nBins-1)/( stats.histMax- stats.histMin);
		stats.binSize = ( stats.histMax- stats.histMin+1)/stats.nBins;
		stats.pixelCount = 0;
		for (int slice=1; slice<=size; slice++) {
			IJ.showStatus("Histogram: "+slice+"/"+size);
			IJ.showProgress((double)slice/size);
			ip = stack.getProcessor(slice);
			for (int y=ry, my=0; y<(ry+rh); y++, my++) {
				int i = y * width + rx;
				int mi = my * rw;
				for (int x=rx; x<(rx+rw); x++) {
					if (mask==null || mask[mi++]==ip.BLACK) {
						v = ip.getPixelValue(x,y);
						stats.pixelCount++;
						sum += v;
						sum2 += v*v;
						histogram[(int)(scale*(v-stats.histMin))]++;
					}
					i++;
				}
			}
		}
		stats.area = stats.pixelCount*pw*ph;
		stats.mean = sum/stats.pixelCount;
		//IJ.write(sum+" "+stats.pixelCount);
		stats.dmode = getMode();
		stats.stdDev = getStdDev(stats.pixelCount, sum, sum2);
		IJ.showStatus("");
		IJ.showProgress(1.0);
	}

	double getMode() {
		int count;
		stats.maxCount = 0;
		for (int i = 0; i < nBins; i++) {
			count = histogram[i];
			if (count > stats.maxCount) {
				stats.maxCount = count;
				stats.mode = i;
			}
		}
		return stats.histMin+stats.mode*stats.binSize;
	}

	double getStdDev(int n, double sum, double sum2) {
		double stdDev;
		if (n>0) {
			stdDev = (n*sum2-sum*sum)/n;
			if (stdDev>0.0)
				stdDev = Math.sqrt(stdDev/(n-1.0));
			else
				stdDev = 0.0;
		}
		else
			stdDev = 0.0;
		return stdDev;
	}
}
